/*
 * tizen_camera_private.c
 *
 * Copyright (c) 2018 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#ifndef __TIZEN_CAMERA_PRIVATE_H__
#define __TIZEN_CAMERA_PRIVATE_H__

#include <linux/videodev2.h>
#include <glib.h>
#include <tbm_bufmgr.h>
#include <tizen-camera.h>

#define CAMERA_HAL_INITIAL_INDEX    -1
#define CAMERA_HAL_INITIAL_FD       -1
#define MESSAGE_CALLBACK_MAX        10
#define PREVIEW_BUFFER_MAX          4
#define VIDEO_BUFFER_MAX            8
#define V4L2_PLANES_MAX             4

typedef struct _camera_hal_handle {
	tbm_bufmgr bufmgr;

	int device_index;
	int device_fd;
	camera_state_t state;
	camera_format_t preview_format;
	camera_format_t video_format;

	/* V4L2 */
	enum v4l2_buf_type v4l2_type;
	uint8_t plane_num;
	int live_buffer_num;

	/* thread */
	GThread *preview_thread;
	GThread *message_thread;

	/* preview callback */
	camera_preview_frame_cb preview_cb;
	void *preview_cb_data;
	gboolean preview_cb_run;
	uint32_t preview_buffer_num;
	camera_buffer_t preview_buffer[PREVIEW_BUFFER_MAX];

	/* video callback */
	camera_video_frame_cb video_cb;
	void *video_cb_data;
	gboolean video_cb_run;
	uint32_t video_buffer_num;
	camera_buffer_t video_buffer[VIDEO_BUFFER_MAX];

	/* message callback */
	camera_message_cb message_cb[MESSAGE_CALLBACK_MAX];
	void *message_cb_data[MESSAGE_CALLBACK_MAX];
	gboolean message_cb_run;
	GQueue *message_list;

	/* thread safe */
	GMutex lock;
	GMutex preview_cb_lock;
	GMutex message_cb_lock;
	GCond preview_cb_cond;
	GCond message_cb_cond;
} camera_hal_handle;

#endif /* __TIZEN_CAMERA_PRIVATE_H__ */
